-- Abstract Syntax for Tree Intermediate Language
module AbsSyn where

type Label = String
type Register = String

data Exp 
  = CONST { value :: Integer }
  | NAME  { name :: Label }
  | TEMP  { temp :: Register }
  | BINOP { op :: BinOp, left :: Exp, right :: Exp }
  | MEM   { addr :: Exp }
  | CALL  { func :: Exp, args :: [Exp] }
  | ESEQ  { stm :: Stm, res :: Exp }
 deriving (Read,Show,Eq)

data Stm
  = MOVE  { dest :: Exp, src :: Exp }
  | EXP   { exp  :: Exp }
  | JUMP  { dest :: Exp, poss :: [Label] }
  | CJUMP { rel :: RelOp, lft :: Exp, rght :: Exp, trueLab :: Label, falseLab :: Label }
  | SEQ   { first :: Stm, second :: Stm }
  | LABEL { label :: Label }
 deriving (Read,Show,Eq)

data Fct = Fct Label [Param] (Maybe Integer){-framesize-} [Stm]
 deriving (Read,Show,Eq)

data BinOp
  = PLUS | MINUS | MUL | DIV | AND | OR | LSHIFT | RSHIFT | ARSHIFT | XOR
 deriving (Read,Show,Eq)

data RelOp
  = EQ | NE | LT | GT | LE | GE | ULT | ULE | UGT | UGE
 deriving (Read,Show,Eq)

data Param = LOC Integer
           | REG Register
           | ADR Integer
 deriving (Read,Show,Eq)
 