-- Time-stamp: <Tue Dec 04 2007 14:22:07 Stardate: [-29]8607.78 hwloidl>
-- $Id: PPIntm.hs,v 1.2 2007/12/04 14:20:49 hwloidl Exp $
--
-- Pretty printing of AST tree for the parsed Intm language file.
---------------------------------------------------------------------------

module PPIntm where

import Maybe
import AbsSyn
import Text.PrettyPrint

dot     = text "."
ass     = text "<-"

render = Text.PrettyPrint.render

class Renderable a where
 pp :: a -> Doc
 
instance Renderable Fct where
 pp (Fct l params fsOpt stms) = text "FCT" <+> text l <+> 
                          parens (hsep (punctuate comma (map pp params))) <+> (if isJust fsOpt then integer (fromJust fsOpt) else text "") <+> lbrace $$ 
                          vcat (map pp stms) $$
                          rbrace

instance Renderable Exp where
 pp (CONST n) = text (show n)
 pp (NAME n) = ppString n
 pp (TEMP t) = ppString t
 pp (BINOP op l r) = pp l <> pp op <> pp r
 pp (MEM a) = text "MEM" <> parens (pp a)
 pp (CALL f args) = text "CALL" <+> pp f <> parens (hsep (punctuate comma (map pp args)))
 pp (ESEQ stm exp) = lbrace <> pp stm <> comma <> pp exp <> rbrace

instance Renderable Stm where
 pp (MOVE d s) = pp d <+> ass <+> pp s
 pp (EXP e) = pp e
 pp (JUMP j js) = text "JUMP" <+> pp j -- <+> lbrack <> pp js <> rbrack
 pp (CJUMP op l r tL eL) = text "CJUMP" <+> (parens (pp l <> pp op <> pp r)) <+> text "?" <+> ppString tL <+> colon <+> ppString eL
 pp (SEQ x xs) = pp x $$ pp xs
 pp (LABEL l) = ppString l <> text ":"

instance Renderable BinOp where
 pp  AbsSyn.PLUS = text "+"
 pp  AbsSyn.MINUS = text "-"
 pp  AbsSyn.MUL = text "*"
 pp  AbsSyn.DIV = text "/"
 pp  AbsSyn.AND = text "&&"
 pp  AbsSyn.OR = text "||"
 pp  AbsSyn.LSHIFT = text "<<"
 pp  AbsSyn.RSHIFT = text ">>"
 pp  AbsSyn.ARSHIFT = text "<<#"
 pp  AbsSyn.XOR = text "^^"


instance Renderable RelOp where
 pp  AbsSyn.EQ = text "=="
 pp  AbsSyn.NE = text "!="
 pp  AbsSyn.LT = text "<"
 pp  AbsSyn.GT = text ">"
 pp  AbsSyn.LE = text "<="
 pp  AbsSyn.GE = text ">="
 pp  AbsSyn.ULT = text "<#"
 pp  AbsSyn.ULE = text "<=#"
 pp  AbsSyn.UGT = text ">#"
 pp  AbsSyn.UGE = text ">#"

instance Renderable Param where
 pp (LOC i) = text "LOC" <+> integer i
 pp (REG t) = text "REG" <+> text t
 pp (ADR i) = text "ADR" <+> integer i

{-
instance Renderable [Expression] where
 pp ss = punctuate "," ss
-}

instance Renderable [Exp] where
 pp ss = hcat (map pp ss)

instance Renderable [Stm] where
 pp ss = vcat (map pp ss)

instance Renderable [Fct] where
 pp ss = vcat (map pp ss)

instance Renderable Char where
 pp c = text (show c)

instance Renderable Int where
 pp n = int n

ppString str = text str